@@ -5,7 +5,7 @@
 ##  BitVectorDemo.pl
 
 use strict;
-use Algorithm::BitVector;
+use Algorithm::BitVector 1.23;
 
 # Construct an EMPTY bitvector (a bitvector of size 0):
 print "\nConstructing an EMPTY bitvector (a bitvector of size 0):\n";
