@@ -21,7 +21,7 @@ use Math::BigInt;
 use Math::Random;
 use Fcntl 'SEEK_CUR';
 
-our $VERSION = '1.22';
+our $VERSION = '1.23';
 
 use overload  '+'        =>    '_join',
               '""'       =>    '_str',
@@ -1185,8 +1185,12 @@ bit arrays and for logical and arithmetic operations on such arrays.
 
 =head1 CHANGES
 
-Version 1.22 removes the Perl version constraints from the module and the
-C<Makefile.PL> files and the C<PREREQ_PM> constraints from the C<Makefile.PL> file.
+Version 1.23 mentions the required modules in the C<Makefile.PL> file but with no
+minimum version numbers.  Additionally, the documentation associated with the methods
+was significantly upgraded in this version.
+
+Version 1.22 removes the Perl version restriction from the module and the
+C<Makefile.PL> files and the C<PREREQ_PM> restrictions from the C<Makefile.PL> file.
 
 Version 1.21 fixes a bug in the code for the Miller-Rabin primality test function
 C<test_for_primality()>.  This version also places a hard limit on the size of the
@@ -1368,8 +1372,8 @@ is C<int($bv1)> is less than C<int($bv2)>.
 Since Perl does not expect these two operators to be invoked in a void context, such
 statements in your code will elicit a warning from Perl to that effect. If these
 warnings annoy you, you can turn them off by surrounding such statements with C<no
-warnings "void";> and C<use warnings;> pragmas.  The other option is to invoke such
-statements in the following manner:
+warnings "void";> and C<use warnings;> directives.  The other option is to invoke
+such statements in the following manner:
 
     $bv1 = $bv1 << $n;
     $bv2 = $bv1 >> $n; 
@@ -1479,13 +1483,16 @@ vector.
 
 =over 4
 
-When you construct bitvectors by block scanning a disk file, you can call this
-method after you are done to close the file handle that was created to read the file:
+When you construct bitvectors by block scanning a disk file, after you are done, you
+can call this method to close the file handle that was created to read the file:
 
-    $bv = Algorithm::BitVector->new( filename => 'testinput1.txt' );
-    ## code to read the blocks of bits for constructing bitvectors
+    $bv = Algorithm::BitVector->new( filename => 'testinput.txt' );
+    ## your code to read bit blocks for constructing bitvectors goes here
     $bv->close_file_handle();
 
+The constructor call in the first statement opens a file handle for reading the bits.
+It is this file handle that is closed by calling C<close_file_handle()>,
+
 =back
 
 =head3 count_bits()
@@ -1495,7 +1502,8 @@ method after you are done to close the file handle that was created to read the
     $bv = Algorithm::BitVector->new( intVal => 45, size => 16 );
     print $bv->count_bits();                       # 4
 
-This method returns the number bits set to 1 in a bitvector.
+This method returns an integer value which is the number of bits set to 1 in the
+bitvector on which the method is invoked.
 
 =back
 
@@ -1541,24 +1549,30 @@ C<$bv_copy> will not affect the other.
 =over 4
 
     ($bv1, $bv2) = $bv->divide_into_two();              # say $bv = 0000000000101101
-    print "\n$bv1\n";                                   # 00000000                                 
+    print "$bv1\n";                                     # 00000000                                 
     print "$bv2\n";                                     # 00101101  
 
+Divides an even sized bitvector into two bitvectors, each of size half of the
+bitvector on which this method is invoked. Throws an exception when invoked on a
+bitvector that is not even sized.
+
 =back
 
 =head3 gcd()
 
 =over 4
 
-This method returns the Greatest Common Divisor of the integer values represented by
-the two bitvectors:
+This method uses the Euclid's algorithm to return the Greatest Common Divisor of the
+integer values represented by the two bitvectors.  The following example shows a call
+to C<gcd()> returning the GCD of the integer values of the bitvectors C<$bv1> and
+C<$bv2>.
 
     $bv1 = Algorithm::BitVector->new( bitstring => '01100110' );   # 102                           
     $bv2 = Algorithm::BitVector->new( bitstring => '011010' );     # 26                            
     $gcd = $bv1->gcd( $bv2 );                                      # 10
     print int($gcd);                                               # 2
 
-Note that the result returned by C<gcd()> is a bitvector.
+The result returned by C<gcd()> is a bitvector.
 
 =back
 
@@ -1570,7 +1584,7 @@ Note that the result returned by C<gcd()> is a bitvector.
     $bv = $bv->gen_random_bits(16);                        # 1100111001010101
 
 The call to C<gen_random_bits()> returns a bitvector whose bits are randomly
-generated.  The number of bits in the returned vector equals the argument integer.
+generated.  The number of bits in the returned bitvector equals the argument integer.
 
 =back
 
@@ -1593,14 +1607,16 @@ index standing for the last (meaning the right-most) bit in the vector:
     print $bv->get_bit(-5);                          # 1   (the first bit)
 
 The C<get_bit()> can also return a slice of a bitvector if the argument to the
-method is an anonymous array of the index range you desire, as in
+method is an anonymous array of the index range you desire, as in the second 
+statement below:
 
     $bv = Algorithm::BitVector->new( bitstring => "10111011");
     my $arr = $bv->get_bit( [3..7] );
     print "@$arr\n";                                 # 1 1 0 1 1 
 
 In this example, we want C<get_bit()> to return all bits at positions indexed 3
-through 7, both ends inclusive.
+through 7, both ends inclusive.  Note that the slice is returned as an array of bits.
+
 
 =back
 
@@ -1614,6 +1630,9 @@ hex representation of the bit pattern:
     $bv = Algorithm::BitVector->new(bitstring => "0110100001100101011011000110110001101111");
     print $bv->get_hex_string_from_bitvector();             # 68656c6c6f
 
+The hex representation is returned in the form if a string of hex characters.  This
+method throws an exception if the size of the bitvector is not a multiple of 4.
+
 =back
 
 =head3 get_text_string_from_bitvector()
@@ -1624,7 +1643,16 @@ hex representation of the bit pattern:
     print "$bv\n";                              # 0110100001100101011011000110110001101111 
     print $bv->get_text_from_bitvector();                           # hello                        
 
-This method throws an exception if the size of the bit pattern is not a multiple of 8.
+The method returns a string of ASCII characters by converting successive 8-bit slices
+of the bitvector into an ASCII character. It throws an exception if the size of the
+bit pattern is not a multiple of 8.  Calling this method to create a text-based print
+representation of a bit vector makes sense only if you don't expect to see any
+unprintable characters in the successive 8-bit slices of the bitvector.  Let's say
+you have created a bitvector from a text string using the appropriate constructor
+call.  Subsequently, you encrypted this text string.  Next, you or someone else
+decrypts the encrypted bit stream.  Since what comes out at the decryption end must
+be the original text string, it would make sense to invoke this method to recover the
+original text.
 
 =back
 
@@ -1646,7 +1674,8 @@ What this example illustrates is dividing the bitvector C<$a> by the modulus bit
 vector C<$mod>.  For a more general division of one bitvector C<$a> by another bit
 vector C<$b>, you would carry out a multiplication of C<$a> by the MI of C<$b>, where
 MI stands for "multiplicative inverse" as returned by a call to the method
-C<gf_MI()>.
+C<gf_MI()>.  A call to C<gf_divide_by_modulus()> returns two bitvectors, one for the
+quotient and the other for the remainder.
 
 =back
 
@@ -1663,6 +1692,8 @@ the value of C<n>:
     $a = Algorithm::BitVector->new( bitstring => '00110011' );
     print $a->gf_MI($modulus, $n);                     # 01101100                                  
 
+Note that the multiplicative inverse is returned as a bitvector.
+
 =back
 
 =head3 gf_multiply()
@@ -1678,7 +1709,8 @@ values in C<GF(2)>, this method returns the product polynomial.
     print $a->gf_multiply($b);                                #00010100110
 
 As you would expect, in general, the bit pattern returned by this method will be
-longer than the lengths if the two operand bitvectors.
+longer than the lengths of the two operand bitvectors.  The result returned by the
+method is in the form of a bitvector.
 
 =back
 
@@ -1696,7 +1728,8 @@ must supply it the bitvector for the modulus polynomial and the value of C<n>.
     print $a->gf_multiply_modular($b, $modulus, $n);         # 10100110                            
 
 This example returns the product of the bit patterns C<$a> and C<$b> modulo the bit
-pattern C<$modulus> in C<GF(2^8)>.
+pattern C<$modulus> in C<GF(2^8)>.  The result returned by the method is in the form
+of a bitvector.
 
 =back
 
@@ -1712,7 +1745,8 @@ the same size.
     print $bv1->hamming_distance( $bv2 );                            # 4
 
 This distance returns the number of bit positions in which two the bit patterns
-differ.
+differ.  The method throws an exception if the two bitvectors are not of the same
+length.  The value returned is an integer.
 
 =back
 
@@ -1772,6 +1806,8 @@ coefficient:
     $bv2 = Algorithm::BitVector->new( bitstring => '10101011' );
     print $bv1->jaccard_distance( $bv2 );                         # 0.375
 
+The value returned by the method is a floating point number between 0 and 1.
+
 =back
 
 =head3 jaccard_similarity()
@@ -1785,6 +1821,8 @@ pointed to by C<$bv1> and C<$bv2>:
     $bv2 = Algorithm::BitVector->new( bitstring => '10101011' );
     print $bv1->jaccard_similarity( $bv2 );                       # 0.675
 
+The value returned by the method is a floating point number between 0 and 1.
+
 =back
 
 =head3 length()
@@ -1819,8 +1857,8 @@ described earlier in this API.
         print "No multiplicative inverse in this case\n";
     }
 
-What that says is that the multiplicative inverse of 17 modulo 32 is 17. That is
-because 17 times 17 in modulo 32 arithmetic equals 1.  When using this method, you
+What this example says is that the multiplicative inverse of 17 modulo 32 is 17. That
+is because 17 times 17 in modulo 32 arithmetic equals 1.  When using this method, you
 must test the value returned for 0.  If the returned value is 0, that means that the
 number corresponding to the bitvector on which this method is invoked does not
 possess a multiplicative inverse with respect to the modulus.
@@ -1851,6 +1889,10 @@ You can pad a bitvector from the left with a designated number of zeros:
     $bv = Algorithm::BitVector->new( bitstring => '101010' );
     print $bv->pad_from_left( 4 );                               # 0000101010
 
+The method returns the bitvector on which it is invoked.  So you can think of it as
+an in-place extension of a bitvector (although, under the hood, the extension is
+carried out by giving a new longer C<_vector> attribute to the bitvector object).
+
 =back
 
 =head3 pad_from_right()
@@ -1862,6 +1904,11 @@ You can pad a bitvector from the right with a designated number of zeros:
     $bv = Algorithm::BitVector->new( bitstring => '101010' );
     print $bv->pad_from_right( 4 );                              # 1010100000
 
+The method returns the bitvector on which it is invoked.  So you can think of it as
+an in-place extension of a bitvector (although, under the hood, the extension is
+carried out by giving a new longer C<_vector> attribute to the bitvector object).
+
+
 =back
 
 =head3 permute()
@@ -1875,6 +1922,8 @@ You can permute the bits in a bitvector with a permutation list as shown below:
     $bv2 =  $bv1->permute( [3, 2, 1, 0, 7, 6, 5, 4] );
     print $bv2;                                                   # 00111101
 
+The method returns a new bitvector with permuted bits.
+
 =back
 
 =head3 rank_of_bit_set_at_index()
@@ -1888,7 +1937,8 @@ of bits that are set up to the argument position, as in
     print $bv->rank_of_bit_set_at_index( 10 );                    # 6
 
 The value 6 returned by this call to C<rank_of_bit_set_at_index()> is the number of
-bits set up to the position indexed 10 (including that position).
+bits set up to the position indexed 10 (including that position).  The method throws
+an exception if there is no bit set at the argument position.
 
 =back
 
@@ -1903,7 +1953,7 @@ Subsequent invocations of C<read_bits_from_file($n)> on this variable will read
 blocks of C<$n> bits and return a bitvector for each block thus read.  The variable
 C<$n> must be a multiple of 8 for this to work.
 
-    $bv = Algorithm::BitVector->new( filename => 'testinput1.txt' );
+    $bv = Algorithm::BitVector->new( filename => 'testinput.txt' );
     $bv1 = $bv->read_bits_from_file(64);
     $bv2 = $bv->read_bits_from_file(64);
     ...
@@ -1914,7 +1964,7 @@ When reading a file as shown above, you can test the attribute C<more_to_read> o
 bitvector object in order to find out if there is more to be read in the file.  The
 C<while> loop shown below reads all of a file in 64-bit blocks. 
 
-    $bv = Algorithm::BitVector->new( filename => 'testinput4.txt' );
+    $bv = Algorithm::BitVector->new( filename => 'testinput.txt' );
     while ($bv->{more_to_read}) {
         my $bv_read = $bv->read_bits_from_file( 64 );
         print "$bv_read\n";
@@ -1942,6 +1992,9 @@ calling this method:
     $bv->reset(0);
     print $bv;                                                  # 00000000  
 
+What the method accomplishes can be thought of as in-place resetting of the bits. The
+method does not return anything.
+
 =back
 
 =head3 reverse()
@@ -1954,6 +2007,9 @@ sense that what was left-to-right earlier now becomes right-to-left, as in
     $bv = Algorithm::BitVector->new( bitstring => '01100001' );
     print $bv->reverse();                                       # 10000110                         
 
+A call to this method returns a new bitvector whose bits are in reverse order in
+relation to the bits in the bitvector on which the method is called.
+
 =back
 
 =head3 runs()
@@ -1981,11 +2037,11 @@ the bit position index.  The method takes two explicit arguments, the first for
 position of the bit you want to set and the second for the value of the bit.
 
     $bv = Algorithm::BitVector->new( bitstring => '1111' );
-    $bv->set_bit(0,0);
-    $bv->set_bit(1,0);
+    $bv->set_bit(0,0);                                # set the first bit to 0
+    $bv->set_bit(1,0);                                # set the next bit to 0
     print $bv;                                        # 0011                                       
-    $bv->set_bit(-1,0);
-    $bv->set_bit(-2,0);
+    $bv->set_bit(-1,0);                               # set the last bit to 0
+    $bv->set_bit(-2,0);                               # set the bit before the last bit to 0
     print $bv;                                        # 0000       
 
 =back
@@ -2002,13 +2058,17 @@ constructed bitvector:
     $bv->set_value( intVal => 45 );
     print $bv;                                 # 101101    
 
+You can think of this method as carrying out an in-place resetting of the bit array
+in a bitvector.  The method does not return anything.
+
 =back
 
 =head3 shift_left()
 
 =over 4
 
-If you want to shift a bit non-circularly to the left, this is the method to call:
+If you want to shift a bitvector non-circularly to the left, this is the method to
+call:
 
     $bv = Algorithm::BitVector->new( bitlist => [1,1, 1, 0, 0, 1] );
     $bv->shift_left(3);
@@ -2017,13 +2077,19 @@ If you want to shift a bit non-circularly to the left, this is the method to cal
     $bv->shift_left(3)->shift_right(3);
     print $bv;                                                  # 000001 
 
+As the bitvector is shifted non-circularly to the left, the exposed bit positions on
+the right are filled with zeros.  Note that the method returns the bitvector object
+on which it is invoked.  That is the reason why the chained invocation of the method
+in the fifth statement above works.
+
 =back
 
 =head3 shift_right()
 
 =over 4
 
-If you want to shift a bit non-circularly to the right, this is the method to call:
+If you want to shift a bitvector non-circularly to the right, this is the method to
+call:
 
     $bv = Algorithm::BitVector->new( bitlist => [1,1, 1, 0, 0, 1] );
     $bv->shift_right(3);
@@ -2032,14 +2098,20 @@ If you want to shift a bit non-circularly to the right, this is the method to ca
     $bv->shift_right(3)->shift_right(2);
     print $bv;                                                  # 000001 
 
+As the bitvector is shifted non-circularly to the right, the exposed bit positions on
+the left are filled with zeros.  Note that the method returns the bitvector object
+on which it is invoked.  That is the reason why the chained invocation of the method
+in the fifth statement above works.
+
 =back
 
 =head3 test_for_primality()
 
 =over 4
 
-If the integer value of a bitvector is small, you can use this method to test it for
-its primality by the Miller-Rabin probabilistic test:
+If the integer value of a bitvector is small (meaning smaller than C<< 0x7f_ff_ff_ff >>),
+you can use this method to test it for its primality through the Miller-Rabin
+probabilistic test:
 
     $p = 7417;
     $bv = Algorithm::BitVector->new( intVal => $p );
@@ -2047,6 +2119,10 @@ its primality by the Miller-Rabin probabilistic test:
     print "The primality test for $p: $check\n";
             # The primality test for 7417: is a prime with probability 0.99993896484375 
 
+The method returns one of two strings: If the primality test succeeds, the method
+returns a string like "C<is a prime with probability xxxxx>".  And if the test fails,
+the method returns the string "C<is NOT a prime>".
+
 =back
 
 =head3 unpermute()
@@ -2063,6 +2139,10 @@ as shown below:
     $bv3 = $bv2->unpermute( [3, 2, 1, 0, 7, 6, 5, 4] );
     print $bv3;                                                   # 11001011
 
+The method returns a new bitvector with unpermuted bits.  Also note that the method
+throws an exception if the permutation list is not as long as the size of the
+bitvector on which the method is invoked.
+
 =back
 
 =head3 write_to_file()
@@ -2086,7 +2166,7 @@ otherwise the bit pattern `00001010' ('\n') in your bitstring will be written ou
 
 =back
 
-=head1 IMPORTS
+=head1 REQUIRED
 
 This module imports the following modules:
 
@@ -2120,10 +2200,10 @@ the string 'BitVector' in the subject line.
 Download the archive from CPAN in any directory of your choice.  Unpack the archive
 with a command that on a Linux machine would look like:
 
-    tar zxvf Algorithm-BitVector-1.22.tar.gz
+    tar zxvf Algorithm-BitVector-1.23.tar.gz
 
 This will create an installation directory for you whose name will be
-C<Algorithm-BitVector-1.22>.  Enter this directory and execute the following commands
+C<Algorithm-BitVector-1.23>.  Enter this directory and execute the following commands
 for a standard install of the module if you have root privileges:
 
     perl Makefile.PL
@@ -2157,9 +2237,8 @@ If I used bash, I'd need to declare:
 The bug in the primality test function, whose fix resulted in Version 1.21, was
 reported by Dana Jacobsen in a bug report filed at C<rt.cpan.org>.  Thanks Dana!
 
-The Perl and the C<PREREQ_PM> constraints were removed in Version 1.22 on Slaven
-Rezic's recommendation.  He says the module runs fine with Perl 5.8.9.  Thanks
-Slaven!
+The restriction on the Perl version was removed on Slaven Rezic's recommendation.  He
+says the module runs fine with Perl 5.8.9.  Thanks Slaven!
 
 =head1 AUTHOR
 
